@@ -1,6 +1,6 @@
 /* util.h - various utility functions
  *
- * Copyright (C) 2005 Gerhard Hring <gh@ghaering.de>
+ * Copyright (C) 2005-2006 Gerhard Hring <gh@ghaering.de>
  *
  * This file is part of pysqlite.
  *
@@ -26,14 +26,13 @@
 #include "Python.h"
 #include "pythread.h"
 #include "sqlite3.h"
+#include "connection.h"
 
-void pysqlite_sleep(double seconds);
-
-int _sqlite_step_with_busyhandler(sqlite3_stmt* statement, void* connection);
+int _sqlite_step_with_busyhandler(sqlite3_stmt* statement, pysqlite_Connection* connection);
 
 /**
- *  * Checks the SQLite error code and sets the appropriate DB-API exception.
- *   * Returns the error code (0 means no error occured).
- *    */
-int _seterror(sqlite3* db);
+ * Checks the SQLite error code and sets the appropriate DB-API exception.
+ * Returns the error code (0 means no error occured).
+ */
+int _pysqlite_seterror(sqlite3* db);
 #endif
